@@ -1,5 +1,5 @@
 
-/*  $Header: /cvs/gnome/gcalctool/gcalctool/get.c,v 1.30 2004/01/22 17:50:58 richb Exp $
+/*  $Header: /cvs/gnome/gcalctool/gcalctool/get.c,v 1.37 2004/07/19 21:13:16 richb Exp $
  *
  *  Copyright (c) 1987-2004 Sun Microsystems, Inc. All Rights Reserved.
  *           
@@ -246,7 +246,7 @@ init_constant(int n, gchar *value)
 {
     gchar *str = g_strdup(value);
 
-    MPstr_to_num(str, DEC, TRUE, v->MPcon_vals[n]);
+    MPstr_to_num(str, DEC, v->MPcon_vals[n]);
     g_free(str);
 }
 
@@ -382,6 +382,24 @@ read_resources()    /* Read all possible resources from the database. */
         }
     }
 
+    /* Set expression (arithmetic precedence) mode as default. */
+    v->syntax = exprs;
+
+    if (get_str_resource(R_SYNTAX, str)) {
+      for (i = 0; i < MAXSYNTAX; i++) {
+	if (EQUAL(str, Rsstr[i])) {
+	  break;
+	}
+      }
+      
+      if (i == MAXSYNTAX) {
+	FPRINTF(stderr, _("%s: invalid syntax mode [%s]\n"), 
+		v->progname, str);
+      } else {
+	v->syntax = i;
+      }
+    }
+
     if (get_bool_resource(R_REGS, &boolval)) {
         v->rstate = boolval;
     }
@@ -393,6 +411,7 @@ read_resources()    /* Read all possible resources from the database. */
     if (get_bool_resource(R_TSEP, &boolval)) {
         v->show_tsep = boolval;
     }
+
 }
 
 
